from . import __define_unit
from ..units import Quantity

quetta = Quantity(1e30, units=1, custom_string="Q", expect_self=True)
ronna  = Quantity(1e27, units=1, custom_string="R", expect_self=True)
yotta  = Quantity(1e24, units=1, custom_string="Y", expect_self=True)
zetta  = Quantity(1e21, units=1, custom_string="Z", expect_self=True)
exa    = Quantity(1e18, units=1, custom_string="E", expect_self=True)
peta   = Quantity(1e15, units=1, custom_string="P", expect_self=True)
tera   = Quantity(1e12, units=1, custom_string="T", expect_self=True)
giga   = Quantity(1e9, units=1, custom_string="G", expect_self=True)
mega   = Quantity(1e6, units=1, custom_string="M", expect_self=True)
kilo   = Quantity(1e3, units=1, custom_string="k", expect_self=True)
hecto  = Quantity(1e2, units=1, custom_string="h", expect_self=True)
deca   = Quantity(1e1, units=1, custom_string="da", expect_self=True)
deci   = Quantity(1e-1, units=1, custom_string="d", expect_self=True)
centi  = Quantity(1e-2, units=1, custom_string="c", expect_self=True)
milli   = Quantity(1e-3, units=1, custom_string="m", expect_self=True)
micro  = Quantity(1e-6, units=1, custom_string="µ", expect_self=True)
nano   = Quantity(1e-9, units=1, custom_string="n", expect_self=True)
pico   = Quantity(1e-12, units=1, custom_string="p", expect_self=True)
femto  = Quantity(1e-15, units=1, custom_string="f", expect_self=True)
atto   = Quantity(1e-18, units=1, custom_string="a", expect_self=True)
zepto  = Quantity(1e-21, units=1, custom_string="z", expect_self=True)
yocto  = Quantity(1e-24, units=1, custom_string="y", expect_self=True)
ronto  = Quantity(1e-27, units=1, custom_string="r", expect_self=True)
quecto = Quantity(1e-30, units=1, custom_string="q", expect_self=True)
